unit oTermComponent;

interface

uses
  uXML,
  ComCtrls,
  XMLIntf,
  SysUtils,
  Classes,
  iCPTerminology;

type
  TTermComponent = class(TInterfacedObject, ITermComponent)
  private
    fParentTerm: ITerm;
    fComponentTerm: ITerm;
    fComponentType: TTermComponentType;
    fComponentOrder: integer;
    fComponentDescription: string;
  public
    constructor Create(aComponentType: TTermComponentType; aParentTerm, aComponentTerm: ITerm; aOrder: integer = 0);
    destructor Destroy; override;

    function getParentTerm: ITerm; stdcall;
    function getComponentTerm: ITerm; stdcall;
    function getComponentType: TTermComponentType; stdcall;
    function getComponentOrder: integer; stdcall;
    function getComponentDescription: string; stdcall;
    function getActive: boolean; stdcall;
    function getComponentDisplayName: string; virtual; stdcall;
    function getDetailLine: string; virtual; stdcall;

    procedure setComponentOrder(const aValue: integer); stdcall;
    procedure setComponentDescription(const aValue: string); stdcall;

    procedure Validate; virtual; stdcall;
    procedure Save(aXMLNode: IXMLNode); virtual; stdcall;
    procedure Load(aXMLNode: IXMLNode); virtual; stdcall;

    function AddToList(aList: TStrings; aUseDisplayName: boolean = False): integer; stdcall;
    function AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode; stdcall;
    function AddToListView(aListView: TListView): TListItem; virtual; stdcall;
  end;

implementation

const
  {Default XML Tags for the properties}
  XML_COMPONENT_TERM = 'COMPONENT_TERM';
  XML_COMPONENT_TYPE = 'COMPONENT_TYPE';
  XML_COMPONENT_ORDER = 'COMPONENT_ORDER';
  XML_COMPONENT_DESCRIPTION = 'COMPONENT_DESCRIPTION';

  {Default values for the properties}
  DEFAULT_COMPONENT_ORDER = 0;
  DEFAULT_COMPONENT_DESCRIPTION = 'No Description';

constructor TTermComponent.Create(aComponentType: TTermComponentType; aParentTerm, aComponentTerm: ITerm; aOrder: integer = 0);
begin
  fComponentType := aComponentType;
  fParentTerm := aParentTerm;
  fComponentTerm := aComponentTerm;
  fComponentOrder := DEFAULT_COMPONENT_ORDER;
  fComponentDescription := DEFAULT_COMPONENT_DESCRIPTION;
end;

destructor TTermComponent.Destroy;
begin
  inherited;
end;

procedure TTermComponent.Load(aXMLNode: IXMLNode);
begin
  try
    fComponentOrder := XMLValueAsInt(aXMLNode, XML_COMPONENT_ORDER, DEFAULT_COMPONENT_ORDER);
    fComponentDescription := XMLValueAsStr(aXMLNode, XML_COMPONENT_DESCRIPTION, DEFAULT_COMPONENT_DESCRIPTION);
  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to load %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

procedure TTermComponent.Save(aXMLNode: IXMLNode);
begin
  try
    XMLComment(aXMLNode, Format('Attributes loaded/saved by %s', [ClassName]));
    if Assigned(fComponentTerm) then
      aXMLNode.ChildValues[XML_COMPONENT_TERM] := fComponentTerm.getID;
    aXMLNode.ChildValues[XML_COMPONENT_TYPE] := fComponentType;
    aXMLNode.ChildValues[XML_COMPONENT_ORDER] := fComponentOrder;
    aXMLNode.ChildValues[XML_COMPONENT_DESCRIPTION] := fComponentDescription;
  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to save %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

function TTermComponent.AddToList(aList: TStrings; aUseDisplayName: boolean = False): integer;
begin
  if aUseDisplayName then
    Result := aList.AddObject(getComponentDisplayName, Self)
  else
    Result := aList.AddObject(fComponentTerm.getTerm, Self);
end;

function TTermComponent.AddToListView(aListView: TListView): TListItem;
begin
  Result := aListView.Items.Add;
  with Result do
    begin
      Caption := fComponentTerm.getTerm;
      Data := Self;
    end;
end;

function TTermComponent.AddToTreeView(aTreeView: TTreeView; aTreeNode: TTreeNode): TTreeNode;
begin
  Result := aTreeView.Items.AddChildObject(aTreeNode, fComponentTerm.getTerm, Self);
end;

{Private Getters}

function TTermComponent.getActive: boolean;
begin
  Result := fComponentTerm.getActive;
end;

function TTermComponent.getComponentDescription: string;
begin
  Result := fComponentDescription;
end;

function TTermComponent.getComponentOrder: integer;
begin
  Result := fComponentOrder;
end;

function TTermComponent.getComponentTerm: ITerm;
begin
  Result := fComponentTerm;
end;

function TTermComponent.getComponentType: TTermComponentType;
begin
  Result := fComponentType;
end;

function TTermComponent.getDetailLine: string;
begin
  Result := 'Base component type - No getDetailLine property defined.';
end;

function TTermComponent.getComponentDisplayName: string;
begin
  Result := 'Base component type - No ComponentDisplayName property defined.';
end;

function TTermComponent.getParentTerm: ITerm;
begin
  Result := fParentTerm;
end;

{ Private Setters }

procedure TTermComponent.setComponentDescription(const aValue: string);
begin
  fComponentDescription := aValue;
end;

procedure TTermComponent.setComponentOrder(const aValue: integer);
begin
  fComponentOrder := aValue;
end;

procedure TTermComponent.Validate;
begin
  if Assigned(fComponentTerm) then
    if not fComponentTerm.getActive then
      fParentTerm.NewError(etLogic, Format('Inactive %s ''%s'' assigned.', [TERM_TYPE[fComponentTerm.getTermType], fComponentTerm.getTerm]));
end;

end.

